!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%% ChoCoLaT vII.1.77 %%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%% Laser Target Charg'omatic Code %%%% version II.1.77 %%%%
!%%%%%%%%%%%%%% for laser-mater interaction %%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%% Developped at ENS during years 2016-1018 %%%%%%%%%%
!%%%%%%%%%%%%% by Alexandre Poyé %%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%% alexandre.poye@gmail.com %%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%% Please contact for further developments %%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%% or updates %%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!Program goal: solving equations for the evolution of a hot electron cloud in a thick target and calculating the amount of electron ejected.
!Tree physical phenomena are included : hot electron cooling by target collision, hot electron ejection, hot electron creation by laser.

!program input argument : I4 interger used as a file number.
!input data from input file: 'input_****.txt' containing :
! % metal   =  I4                    (1:carbone 2:aluminum 3:cuivre 4:tungstene 5:tantale 6:plomb)
! % N_iks   =  I6                    (number of point for the distribution function discretisation in mec² )
! % dt      =  E11.4                 (time step for initial value solver, in ps)
! % Nb_Th     =  I4                  (energy maximum of the distribution function as Nb_th * Th (eq:B1) )
! % diag_s    =  I4                  (1 save every diag_s iterations for the scalar diagnostic. 0 is setting the diagnostic off )
! % diag_f    =  I4                  (1 save every diag_h iterations for the distribution function diagnostic. 0 is setting the diagnostic off )
! % t_laser =  E11.4                 (duration of the laser pulse, in ps)
! % E_laser =  E11.4                 (energy of the laser pulse, in J)                    
! % C_abs   =  E11.4                 (absorbtion coefficent of the laser pulse, from 0 to 1)                    
! % w_laser =  E11.4                 (wavelength of the laser pulse, in µm)                  
! % d_laser =  E11.4                 (diameter of the laser spot, in µm)                    
! % d_targe =  E11.4                 (diameter of the target, in µm)   
! % e_targe =  E11.4                 (thickness of the target, in µm) 


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%% parameters list module %%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
module parameters
implicit none
!electron cloud variables
real*8 :: J        ! total current ejected in kA (eq 16a)
real*8 :: N        ! charge in the hot cloud in nC (eq 11a)
real*8 :: Q        ! charge ejected from the target in nC (eq 16b)
real*8 :: vol      ! average volume of the hot electron cloud in µm^3 (eq 11b)
real*8 :: RLD      ! average ratio between cloud radius and Debye length (used in eq 18)
real*8 :: ne       ! average electron density in the cloud in nC/µm^3 (eq 11a and eq 11b)
real*8 :: LambdaD  ! Debye length in µm (used in eq 17, 19, 20a and 20b)
real*8 :: phi      ! potential to normalised the temperature ( eq 17, 19, 20a, 20b)
real*8 :: eLD      ! ratio between tharget thickness and Debye length (used in eq 19, 20a and 20b)
real*8 :: T_hot_e  ! hot electron cloud in mec² (eq 3)
real*8 :: Rav      ! mean cloud radius in µm (eq 12b)
real*8 :: Em       ! mean energy of f in mec² (eq 12a)
!potential barrier variables
real*8 :: phi_th ! Normalized thermal potential (eq 17 or 19)
real*8 :: phi_E  ! Normalized electrostatic potential (eq 21)
real*8, dimension(:),allocatable :: J_time,R_time,Time !J,R,time saving tables for electrostatic potential calculation (eq 21)
logical :: target_isolated = .true.   ! true : target isolated, phi_E is included. false : target grounded phi_E = 0

!time variables
real*8 :: dt     ! time step in ps.
real*8 :: t      ! time in ps.
real*8 :: t_max  ! maximal time for J_time, R_time and Time array boundary, in ps. Beware, numel = t_max/dt can be out of memory if exagerations.
real*8 :: d_T    ! time step for tee calculation, in ps.

!laser variables
real*8 :: w_laser   ! laser wavelength in µm.
real*8 :: r_laser   ! laser spot radius in µm.
real*8 :: t_laser   ! laser pulse duration in ps.
real*8 :: E_laser   ! laser energy in J.
real*8 :: C_abs     ! absorbtion coefficient.
real*8 :: P_laser   ! laser power in J/ps
real*8 :: I_laser   ! laser intensity in J/ps/µm^2 
real*8 :: a0        ! normalised laser intensity (eq 3)
real*8 :: d_laser   ! laser spot diameter in µm.
real*8, dimension(:), allocatable :: f_0 ! distribution function of the electron generated by the laser in nC/mec² (eq 2).
real*8 :: Em0       ! mean energy of f_0 in mec²


!distribution function arrays
real*8, dimension(:), allocatable :: iks_table      ! normalised relativistic factor
real*8 ::  d_iks  ! energy step for distribution function in mec²
integer :: i_iks  ! counter for energy distribution and energy arrays.
integer :: N_iks  ! Number of energy bin.
integer :: Nb_Th  ! Maximal energy of the energy table is Nb_Th * T_hot_e in mec².  
real*8, dimension(:), allocatable :: f_table        ! distribution function of the number of hot electron in the target in nC/mec² (eq 1).
real*8, dimension(:), allocatable :: gfr_table      ! distribution function of the number of hot electron escaping the target from front side in nC/mec² (eq 1).
real*8, dimension(:), allocatable :: gre_table      ! distribution function of the number of hot electron escaping the target from rear side in nC/mec² (eq 1).

! table of variable for each \epsilon_0 for tee calculation (eq 6).
real*8, dimension(:), allocatable :: v_table        ! speed of hot electron in the target in µm/ps.
real*8, dimension(:), allocatable :: e_table        ! energy of hot electron in the target in mec².
real*8, dimension(:), allocatable :: rmax_table     ! maximal range of hot electron in the target in µm (eq 4)
real*8, dimension(:), allocatable :: r_table        ! range of hot electron in the target in µm (eq 8) (radial)
real*8, dimension(:), allocatable :: z_table        ! range of hot electron in the target in µm (eq 9) (axial)
real*8, dimension(:), allocatable :: sfr_table      ! section of the cloud in µm² (eq 10a)
real*8, dimension(:), allocatable :: vol_table      ! volume of the cloud in µm³ (eq 10b)
real*8, dimension(:), allocatable :: xE_table       ! energetic distance in µm (fig 3)
real*8, dimension(:), allocatable :: C_slow         ! geometric coefficient speed correction (in eq 9)
real*8, dimension(:), allocatable :: thetafr_table  ! scattering angle from the front side (eq 13a and 14a)
integer :: i_phi                                    ! energy index for the potential at front side.
! rear side value for 
real*8, dimension(:), allocatable :: thetare_table  ! scattering angle from the rear side (eq 13b, 13c, 14b and 14c)
real*8, dimension(:), allocatable :: sre_table      ! scattering angle from the rear side (eq 9a)

! table of variable for each \epsilon_0 for tee calculation (eq 6).
real*8, dimension(:), allocatable :: e1_table       ! energy of hot electron in the target in (mec²) (eq 5).
real*8, dimension(:), allocatable :: v1_table       ! speed of hot electron in the target in µm/ps.
real*8, dimension(:), allocatable :: r1_table       ! range of hot electron in the target in µm.
real*8, dimension(:), allocatable :: tee_table      ! cooling time of electron in ps (eq 6)



 
!target variables
integer :: metal      ! kind of metal (1:carbon 2:aluminum 3:copper 4:tungsten 5:tantalum 6:lead)
real*8 :: Z_target    ! Atomic number
real*8 :: rho_target  ! target density g/cm³
real*8 :: A_target    ! Masse number
real*8 :: gg          ! coeficient for electron-ion scattering (fig 3).
real*8 :: cond        ! 0 for insulating material, 1 for conductive one.
real*8 :: d_targe     ! diameter of the target in µm.
real*8 :: e_targe     ! thickness of the target in µm.


! program management variables
integer :: counter                ! counter for the number of iterations
integer :: counter_diag           ! counter for the number of iterations between each scalar diag
integer :: counter_diag_heavy     ! counter for the number of iterations between each distrib function diag
integer :: iteration_diag         ! number of iterations spacing each scalar diag
integer :: iteration_diag_heavy   ! number of iterations spacing each distrib function diag
integer :: i                      ! counter of aguments



real*8 ::  pi= 3.1415926535897932384626433832795028841971693993751
logical :: getout                 ! conditions for the main while loop exit.  
real*8 :: cpu_T1, cpu_T2          ! chronometer time for the execution.
integer*4 :: num_arg,numbfile     ! String for file management.
character(len=22),dimension(:),allocatable :: arg_string     ! String for argument management.
character(len=4) :: char_a        ! String for file management.
character(len=22) :: N_iks_string ! String for distrib diagnostic.

end module parameters
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%% main programme %%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
program charge_target
use parameters
implicit none


!run chronometer start
call CPU_TIME(cpu_T1) 

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%% INITIALISATION %%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%% default parameters %%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

!1: carbon
!2: aluminum
!3: copper
!4: tungsten
!5: tantalum
!6: lead
metal = 5
d_targe = 10000 !in um
e_targe = 30 !in um 

! Resolution parameters
dt = 0.001    ! time evolution resolution
N_iks = 2000  ! number of point for energy arrays
t_max = 500. ! maximal time for simulation.
Nb_Th = 20
iteration_diag = 10
iteration_diag_heavy = 0

! laser parameter
t_laser = 0.1 ! in ps
E_laser = 5.0 ! in J
C_abs   = 0.4 
w_laser = 0.8! in um
d_laser = 12 ! in um



!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%% read input file %%%%%%%%%%%%%%%
!%%%%%%%%%% create output file %%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
call files_manager ! read input files, create output files.

 !argument can be added to the launch command. They replace the input file arguments.
num_arg=command_argument_count() !count arguments
allocate(arg_string(1:num_arg))
if(num_arg > 1)then
  do i = 2,num_arg
    call getarg(i,arg_string(i)) ! read arguments
  enddo
  read (arg_string(2),"(E11.4)") E_laser ! distribute arguments in correspondant variable
  read (arg_string(3),"(E11.4)") d_laser 
!  read (arg_string(4),"(E11.4)") t_laser 
!  read (arg_string(5),"(I1)")    metal  
endif
deallocate(arg_string)
call create_output_file(numbfile)
write(N_iks_string,'(I8)') N_iks ! used for the output write.
call metal_choice  ! fill material parameters with proper values.



!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! calculation of initial temperature due to beg's law. (eq 3)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r_laser = d_laser/2./sqrt(log(2.))
t_laser = t_laser*sqrt(pi)/ (2*sqrt(log(2.)))
P_laser = E_laser/t_laser
I_laser = P_laser  / ( pi* r_laser**2)
a0 = 0.853*w_laser*10*sqrt(I_laser)

if (a0 > 0.03 ) then
  T_hot_e = max(0.47*a0**(2./3.), sqrt(1+a0**2)-1)
else 
  T_hot_e = 3*a0**(4./3.)
endif

if(iteration_diag .NE. 0)then
  write(102, '(A11,E11.4)') '% T_hot_e = ', T_hot_e*511
endif
if(iteration_diag_heavy .NE. 0)then
  write(104, '(A11,E11.4)') '% T_hot_e = ', T_hot_e*511
  write(105, '(A11,E11.4)') '% T_hot_e = ', T_hot_e*511
  write(106, '(A11,E11.4)') '% T_hot_e = ', T_hot_e*511
endif



!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%% initialisation of ChoCoLaT2 %%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! energy resolution (eq B1)
d_iks = T_hot_e*Nb_Th/N_iks

 
!allocating f_distrib tables:
allocate(iks_table(1:N_iks),f_table(1:N_iks),rmax_table(1:N_iks),f_0(1:N_iks), &
         r_table(1:N_iks), r1_table(1:N_iks), vol_table(1:N_iks), &
         e_table(1:N_iks), v_table(1:N_iks),e1_table(1:N_iks), v1_table(1:N_iks), &
         xE_table(1:N_iks), tee_table(1:N_iks), z_table(1:N_iks), C_slow(1:N_iks),&
         sfr_table(1:N_iks), sre_table(1:N_iks), gfr_table(1:N_iks), gre_table(1:N_iks),&
         thetafr_table(1:N_iks),thetare_table(1:N_iks))
! setting all to zero.
thetafr_table = 0.     
thetare_table = 0.
iks_table = 0.
f_table = 0.
e_table = 0.
v_table = 0.
e1_table = 0.
v1_table = 0.
r1_table = 0.
r_table = 0.
rmax_table = 0.
f_0 = 0.
gfr_table = 0.
gre_table = 0.
r_table = 0.
sfr_table = 0.
sre_table = 0.
vol_table = 0.
z_table = 0.
 C_slow = 0.
! electron/ion collision rate (fig 3).
gg = 0.187 * Z_target**(2./3.)

! filling tables for each epsilon.
do i_iks = 1,N_iks
  iks_table(i_iks) = 1+i_iks*d_iks
  !diffusion radius distribution function
  rmax_table(i_iks)= 2.76*10**3 *A_target *(0.511*(iks_table(i_iks)- 1 ))**(5./3.)*&
    Z_target**(-8./9.)*(1 + 1.957*(0.511*(iks_table(i_iks)- 1 )))**(-5./3.) *&
    (1 + 1*(0.511*(iks_table(i_iks)- 1 )+ 10**6*(0.511*(iks_table(i_iks)- 1 ))**4))**(1./8.) / rho_target  ! (eq 4)
    
  v_table(i_iks) = 300*sqrt(iks_table(i_iks)**2.-1)/iks_table(i_iks)

  e_table(i_iks) = (iks_table(i_iks)-1) 

  xE_table(i_iks)= rmax_table(i_iks)*(1+2.*gg-0.21*gg**2.)/(2.*(1+gg)**2.)      ! (fig 3)
  
  C_slow(i_iks) = (rmax_table(i_iks) - min(xE_table(i_iks),e_targe))/rmax_table(i_iks) !(in eq 9)
  
  f_0(i_iks) = exp(-(iks_table(i_iks)-1)/T_hot_e)  ! (eq 2)
enddo
  f_0 =  f_0 / (sum(f_0)*d_iks)             ! normalizing f_0
  Em0 = sum((iks_table-1)*f_0)*d_iks        ! calculating the average energy (equal to T_hot_e also !)


! calulation of t_ee (eq 6): energy loss calculation with time propagation and range reaching for all energy value.
do i_iks = 1,N_iks
  t=0
  d_T = rmax_table(i_iks)/v_table(i_iks)/ 1000
  getout = .false.
  do while (getout .eqv. .false.)
    t=t+d_T
    e1_table(i_iks) = e_table(i_iks)*(1-r1_table(i_iks)/rmax_table(i_iks))**(3./5.)
    v1_table(i_iks) = 300.*sqrt((e1_table(i_iks)+1)**2.-1)/(e1_table(i_iks)+1)
    r1_table(i_iks) = r1_table(i_iks)+d_T*v1_table(i_iks)
    if(r1_table(i_iks) > 0.999*rmax_table(i_iks))then
      tee_table(i_iks) = t
      getout = .true.
    endif
  enddo
enddo


! writting parameters calulated.
if(iteration_diag .NE. 0)then
  write(102, '(A11,E11.4)') '% N_tot   = ', (10**3/0.511) * C_abs * E_laser/Em0
endif
if(iteration_diag_heavy .NE. 0)then
  write(104, '(A11,E11.4)') '% N_tot   = ', (10**3/0.511) * C_abs * E_laser/Em0
  write(105, '(A11,E11.4)') '% N_tot   = ', (10**3/0.511) * C_abs * E_laser/Em0
  write(106, '(A11,E11.4)') '% N_tot   = ', (10**3/0.511) * C_abs * E_laser/Em0
endif
if(iteration_diag .NE. 0)then
  write(102, '(A11,E11.4)') '% t_life  = ', sum(tee_table*f_0)*d_iks
endif
if(iteration_diag_heavy .NE. 0)then
  write(104, '(A11,E11.4)') '% t_life  = ', sum(tee_table*f_0)*d_iks
  write(105, '(A11,E11.4)') '% t_life  = ', sum(tee_table*f_0)*d_iks
  write(106, '(A11,E11.4)') '% t_life  = ', sum(tee_table*f_0)*d_iks
endif

! writing headers of outputs
! writing epsilon values
if(iteration_diag_heavy .NE. 0)then
  write(104,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") 0., e_table*511.
  write(105,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") 0., e_table*511.
  write(106,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") 0., e_table*511.
endif
if(iteration_diag .NE. 0)then
  write(102, '(10A14)') 'time [ps]', 'radius [µm]', 'lambdaD [µm]', 'T   [keV]',&
                      'Q in [nC]', 'pot th [keV]', 'pot E [keV]', 'J fr [kA]', 'J re [kA]', 'Q [nC]'
endif


! initialisating scalars and counters
getout = .false.
t = 0.
i_phi = N_iks+1
J = 0
counter=1
counter_diag=1
f_table = 0.
allocate(J_time(1:ceiling(t_max/dt)),R_time(1:ceiling(t_max/dt)),Time(1:ceiling(t_max/dt)))


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%% MAIN WHILE LOOP START %%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
do while(getout .eqv. .false.)
  t=t+dt        ! one step forward !
  do i_iks = 1,N_iks  

    ! feed f by the laser with electrons issued from f_0
    if(t<t_laser)then 
      f_table(i_iks) =  f_table(i_iks) + dt*( (10**3/0.511) * C_abs * E_laser/Em0/t_laser *f_0(i_iks) ) !(first term of eq. 1)
    endif
    
    r_table(i_iks) = min(rmax_table(i_iks), t*v_table(i_iks))*C_slow(i_iks)      ! (eq 8)
    z_table(i_iks) = min(rmax_table(i_iks), t*v_table(i_iks))                    ! (eq 9)


    ! setting cloud surface, volume, ejection angle correspondingly to target sizes and xi_E (eq 10a-b, 13a-b-c, 14a-b-c).
    if(r_table(i_iks) > d_targe/2.)then
    ! case if target radius too small compare to radial range.
      sfr_table(i_iks) = pi * (d_targe/2.)**2.    
      thetafr_table(i_iks) = pi/2.
      if(z_table(i_iks) > e_targe)then  
        sre_table(i_iks) = sfr_table(i_iks)
        Vol_table(i_iks) = pi * (d_targe/2.)**2. * e_targe 
        thetare_table(i_iks) = pi/2.  
      else
        sre_table(i_iks) = 0.
        Vol_table(i_iks) = pi * (d_targe/2.)**2. * z_table(i_iks)
        thetafr_table(i_iks) = 0.      
      endif 
    else
      sfr_table(i_iks) = pi * (r_laser + r_table(i_iks))**2.
      if(xE_table(i_iks) < e_targe)then
        ! case if thick target, front face.   
        thetafr_table(i_iks) = atan(r_table(i_iks)/xE_table(i_iks))

        ! case if thick target, rear face.     
        if(z_table(i_iks) > e_targe)then
          sre_table(i_iks) = sfr_table(i_iks)
          Vol_table(i_iks) = pi * (r_laser + r_table(i_iks))**2. * e_targe  
          thetare_table(i_iks) =  atan(r_table(i_iks) / ( e_targe - xE_table(i_iks))) 
        else
          sre_table(i_iks) = 0.  
          Vol_table(i_iks) = pi * (r_laser + r_table(i_iks))**2. * z_table(i_iks)         
          thetare_table(i_iks) = 0.
        endif     
      else
        ! case if thin target, front face.
          thetafr_table(i_iks) = atan( r_table(i_iks) / e_targe ) 
        ! case if thin target, rear face.     
        if(z_table(i_iks) > e_targe)then     
          sre_table(i_iks) = sfr_table(i_iks)
          Vol_table(i_iks) = pi * (r_laser + r_table(i_iks))**2. * e_targe
          thetare_table(i_iks) = pi/2.    
        else
          sre_table(i_iks) = 0.
          Vol_table(i_iks) = pi * (r_laser + r_table(i_iks))**2. * z_table(i_iks)
          thetare_table(i_iks) = 0.
        endif   
      endif
    endif
  

    f_table(i_iks) = f_table(i_iks)*max(0., 1.  - dt / tee_table(i_iks) ) !(second term of eq. 1)
    f_table(i_iks) = max(0., f_table(i_iks) - dt*gfr_table(i_iks) - dt*gre_table(i_iks)) !(third and forth term of eq. 1)

  enddo

  ! calul of scalar for the potential
  N = sum(f_table)*d_iks                          ! number of hot electron (eq 11a)
  Em = sum(f_table*e_table)*d_iks / N             ! temperature (eq 12a)
  Rav = sum(f_table*min(r_laser + r_table, d_targe/2.))*d_iks / N  ! cloud radius average (eq 12b)
  vol = sum(f_table*vol_table)*d_iks/N            ! cloud volume average (eq 10b)
  ne  = w_laser**2*(10./(1.6*1.1))* N/vol         ! cloud density normalised to critical density
  LambdaD =  w_laser*sqrt(Em/ne) / (2*pi)         ! hot electron Debyelenght
  RLD = Rav/LambdaD                               ! average electronic range / hot electron Debyelenght
  eLD = e_targe/LambdaD                           ! target thickness / hot electron Debyelenght

  ! calculate the phi barrier
  call phi_calc

  gfr_table = 0.             ! (eq 15a) in kA/mec²
  do i_iks = i_phi,N_iks
    gfr_table(i_iks) = (sin(thetafr_table(i_iks)/2.)**2)*v_table(i_iks) &
                        * f_table(i_iks) * sfr_table(i_iks)/ vol_table(i_iks)
  enddo
  
  gre_table = 0.             ! (eq 15b) in kA/mec²
  do i_iks = i_phi,N_iks
    gre_table(i_iks) = (sin(thetare_table(i_iks)/2.)**2)*v_table(i_iks) &
                        * f_table(i_iks) * sre_table(i_iks)/ vol_table(i_iks)
  enddo

  J= sum(gfr_table + gre_table)*d_iks   ! (eq 16a)
  Q = Q + dt*J                          ! (eq 16b)
   
  !increasing iteration counter for potential calulation. and for diag
  counter = counter+1  

  ! out_of_while conditionners
  if(t>t_max)then !arbitrary time limit
    getout=.true.
    print*, 'time max reached'
  endif

   if(N<0.001*Q .and. t>t_laser)then !hot electron cooling over.
    getout=.true.
    print*, 'empty electron cloud'
  endif 
 
  if(isnan(J))then !ejection total or bug.
    getout=.true.
    print*, 'NaN J reached'
  endif
  
  ! writing scalar diagnotics
  if(iteration_diag .NE. 0)then
    if(counter_diag == iteration_diag )then
      counter_diag =1
      if(.not.getout)then
        write(102, '(11ES14.4E3)') t, Rav, LambdaD, Em*511  &
        , N, phi_th*Em*511, phi_E*Em*511,sum(gfr_table)*d_iks, sum(gre_table)*d_iks, Q
        flush(102)
      endif
    else
      counter_diag = counter_diag+1 
    endif 
  endif
  
  ! writing distrib function diagnotics
  if(iteration_diag_heavy .NE. 0)then
    if(counter_diag_heavy ==  iteration_diag_heavy  )then
      counter_diag_heavy =1
      if(.not.getout)then
        write(104,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") t, f_table/511.
        write(105,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") t, gfr_table/511.
        write(106,FMT= "((E15.4),"//trim(adjustl(N_iks_string))//"(ES14.4E3))") t, gre_table/511.
        flush(104)
        flush(105)
        flush(106)
      endif
    else
      counter_diag_heavy = counter_diag_heavy+1 
    endif 
  endif
  
enddo



!run chronometer stop
call CPU_TIME(cpu_T2) 
print*, 'I found ', Q, ' nC'
print*, ' '

!close input files and output files, deallocate all.
if(iteration_diag .NE. 0)then
  close(101)
  close(102)
endif
if(iteration_diag_heavy .NE. 0)then
  close(104)
  close(105)
  close(106)
endif

! deallocating things.
deallocate(iks_table,f_table,rmax_table,f_0, &
         r_table, r1_table, vol_table, &
         e_table, v_table,e1_table, v1_table, &
         xE_table, tee_table, z_table, C_slow,&
         sfr_table, sre_table, gfr_table, gre_table,&
         thetafr_table,thetare_table)
deallocate(J_time,R_time,Time)

!short screen message with run time
print*, 'I run during',  (cpu_T2 - cpu_T1)/60.,' min'


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!! SUBROUTINES !!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

contains

subroutine Phi_calc
use parameters, only : J,t,dt,counter,pi,t_max,r_laser, phi, N,LambdaD, vol,RLD
use parameters, only : iks_table, N_iks, i_phi, phi_E, phi_th
use parameters, only : cond, d_targe, e_targe, target_isolated
use parameters, only : J_time, R_time, Time
implicit none
real*8 :: c = 300. !speedlight
real*8 :: zero=0.  ! zero.
real*8 :: Rp       ! radius of the electrostatic potential (µm)
integer :: k,iks

!calulation of thermal potential and minimal potential position (eq 18, 19 and 20a-b).
call Phi_th_interp(phi_th,RLD,eLD)

if(target_isolated)then
  !filling table of J, R, time with last values.
  J_time(counter) = J
  R_time(counter) = RLD*LambdaD
  Time(counter) = t

  phi_E = 0 !cumulated contribution of the electrostatic potential from the current and radius at time k between 0 to t.

  ! It is normalised to T. calculated at surface position (eq 21)
  !!conductive target electrostatic potential
  if(cond == 1.)then
    do k=1,counter
      Rp = min(R_time(k) + c*(t - Time(k)),d_targe) ! electrostatic potential radius propagating because charge reorganisation at target surface.
      phi_E = phi_E + J_time(k) *dt * 0.5 *vol / ( pi*Rp*N*LambdaD**2) ! normalized electrostatic potential to T.
    enddo
  
  !!dielectric target electrostatic potential
  elseif( cond == 0.) then
    do k=1,counter
      Rp = R_time(k) ! electrostatic potential radius (no propagation because no conductivity)
      phi_E = phi_E + J_time(k) *dt * 0.5 *vol / ( pi*Rp*N *LambdaD**2) ! normalized electrostatic potential to T.
    enddo    
  endif

  !!summation of potential electrostatic and thermal.
  phi =  phi_E + phi_th
else
  phi =  phi_th  
endif


!research of the barrier index i_phi in the table iks
do iks = 1,N_iks
  if(e_table(iks) .GT. phi*Em) exit
enddo
i_phi = iks-1


end subroutine Phi_calc
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subroutine Phi_th_interp(phi_max_out,RLDin,eLDin)
use parameters, only : t
implicit none
real*8 :: z,l,E, alpha, b, bpE, phi_max
logical, save :: first_time = .true.
real*8,intent(out) :: phi_max_out
real*8,intent(in) :: RLDin, eLDin

! thick thermal formula (eq 18)
phi_max =0
phi_max_out=0
l=sqrt(2.*exp(1.))/RLDin
z = 0.
phi_max_out = (l+z)/(sqrt((l+z)**2.+1.))*log((sqrt((z+l)**2.+1.)*sqrt(z**2.+1.)+1.+z*(z+l))/(l*(sqrt((z+l)**2.+1.)-z-l))) &
                                                  -asinh(z) -log(2.) -2.*log(l+z)+log(l)
                                                  
!thin target phi_th calcul (eq 19 and 20a-b)
if(eLDin < 1.) then
  alpha = 2./eLDin**2. *(sqrt(1.+eLDin**2./2.)-1.)
  E=eLDin/RLDin
  b =2./(eLDin*RLDin*alpha)-E/2.
  bpE = 2./(eLDin*RLDin*alpha)
  phi_max = alpha*RLDin**2.*(E/2.*(sqrt(E**2./4.+1.)-E/2.) +asinh(E/2.)) &
    -2.*( +log(2.) -1. + b/(sqrt(b**2.+1.))*log(sqrt(b**2.+1.)-b)) &
    +2.*(b/(sqrt(b**2.+1.))*log((sqrt(b**2.+1.)*sqrt(E**2./4.+1.)-b*E/2.+1.)/(bpE)) &
    - sqrt(E**2./4.+1.)/bpE &
    - b/bpE - log(bpE) +asinh(E/2.))
  phi_max = phi_max*(1-alpha*eLDin**2./4)

  if(phi_max_out > phi_max) then ! comparison of possible potentials.
    phi_max_out = phi_max
  endif
endif

end subroutine Phi_th_interp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!! list of materials !!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine metal_choice
use parameters, only : Z_target,rho_target, A_target, cond, metal
implicit none

select case (metal)
case(1)
  Z_target   = 6    !carbone
  rho_target = 2.00 !carbone
  A_target   = 12.0 !cardone
  cond       = 0. 
case(2)
  Z_target   = 13   !aluminum
  rho_target = 2.70 !aluminum
  A_target   = 27.0 !aluminum
  cond       = 1. 
case(3)
  Z_target   = 29   !cuivre
  rho_target = 8.96 !cuivre
  A_target   = 63.5 !cuivre
  cond       = 1. 
case(4)
  Z_target   = 74   !tungstene
  rho_target = 19.30!tungstene
  A_target   = 183.8!tungstene
  cond       = 1. 
case(5)
  Z_target   = 73   !tantale
  rho_target = 16.4 !tantale
  A_target   = 180.94 !tantale
  cond       = 1. 
case(6)
  Z_target   = 82    !plomb
  rho_target = 11.35 !plomb
  A_target   = 207 !plomb
  cond       = 1. 
case(7)
    Z_target   = 79   !gold
    rho_target = 19.3 !gold
    A_target   = 196.9 !gold
    cond       = 1. 
  
end select

end subroutine metal_choice
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!! File manager for output !!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine files_manager
use parameters
implicit none
character(len=14) :: file_name_in
logical :: file_exists
!read input file, create output files with check and warnings...

if(iargc()==0)then! check if argument is there.
  print*, 'no input file number. I run with default value and create file 9999'
  numbfile = 9999
else
  call getarg(1,char_a)! check if the input file #### is there.
  read(char_a , *) numbfile
  write(file_name_in , '(A6,A4,A4)') 'input_',char_a,'.txt'
  INQUIRE(FILE=file_name_in, EXIST=file_exists)
  if(file_exists .eqv. .false.)then
    print*, 'I got file number ',char_a,'. But I can''t find the input file.'        
    print*, 'I''ll look for a defautl file 0000'
    write(file_name_in , '(A14)') 'input_0000.txt'
    INQUIRE(FILE=file_name_in, EXIST=file_exists)
    if(file_exists .eqv. .true.)then
      call read_input_file(0000)
    else
      print*, 'No default input file 0000. I will use default parameters.'
    endif
  else
    call read_input_file(numbfile)       
  endif
end if

end subroutine files_manager
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subroutine create_output_file(g)
use parameters
implicit none
LOGICAL :: file_exists=.true.
integer, intent(in) :: g
integer :: f
character(len=23) :: file_name
! create output files and copy input file as reminder.

if(iteration_diag .NE. 0) then
  write(file_name , '(A14,I4.4,A4)') 'output_scalar_',g,'.txt'
  INQUIRE(FILE=trim(file_name), EXIST=file_exists)
  if(file_exists)then
    print*, 'there is already some output here. Change the input number or remove the output file'
    stop
  end if
  open(102, file=file_name, status = 'new', form = 'formatted')
  write(102, '(A11,I4)') '% metal   = ', metal
  write(102, '(A11,I6)') '% N_iks   = ', N_iks 
  write(102, '(A11,E11.4)') '% dt      = ', dt
  write(102, '(A11,I6)') '% Nb Th   = ', Nb_Th
  write(102, '(A11,I6)') '% diag s  = ', iteration_diag
  write(102, '(A11,I6)') '% diag f  = ', iteration_diag_heavy
  write(102, '(A11,E11.4)') '% t_laser = ', t_laser
  write(102, '(A11,E11.4)') '% E_laser = ', E_laser
  write(102, '(A11,E11.4)') '% C_abs   = ', C_abs
  write(102, '(A11,E11.4)') '% w_laser = ', w_laser
  write(102, '(A11,E11.4)') '% d_laser = ', d_laser
  write(102, '(A11,E11.4)') '% d_targe = ', d_targe
  write(102, '(A11,E11.4)') '% e_targe = ', e_targe
  write(102, '(A12,L1)')    '% T_isol  = ', target_isolated 
endif
if(iteration_diag_heavy .NE. 0) then
  write(file_name , '(A14,I4.4,A4)') 'output_f_in___',g,'.txt'
  open(104, file=file_name, status = 'new', form = 'formatted')
  write(104, '(A11,I4)') '% metal   = ', metal
  write(104, '(A11,I6)') '% N_iks   = ', N_iks 
  write(104, '(A11,E11.4)') '% dt      = ', dt
  write(104, '(A11,I6)') '% Nb Th   = ', Nb_Th
  write(104, '(A11,I6)') '% diag s  = ', iteration_diag
  write(104, '(A11,I6)') '% diag f  = ', iteration_diag_heavy
  write(104, '(A11,E11.4)') '% t_laser = ', t_laser
  write(104, '(A11,E11.4)') '% E_laser = ', E_laser
  write(104, '(A11,E11.4)') '% C_abs   = ', C_abs
  write(104, '(A11,E11.4)') '% w_laser = ', w_laser
  write(104, '(A11,E11.4)') '% d_laser = ', d_laser
  write(104, '(A11,E11.4)') '% d_targe = ', d_targe
  write(104, '(A11,E11.4)') '% e_targe = ', e_targe
  write(104, '(A12,L1)')    '% T_isol  = ', target_isolated 
  write(file_name , '(A14,I4.4,A4)') 'output_f_Jfr__',g,'.txt'
  open(105, file=file_name, status = 'new', form = 'formatted')
  write(105, '(A11,I4)') '% metal   = ', metal
  write(105, '(A11,I6)') '% N_iks   = ', N_iks 
  write(105, '(A11,E11.4)') '% dt      = ', dt
  write(105, '(A11,I6)') '% Nb Th   = ', Nb_Th
  write(105, '(A11,I6)') '% diag s  = ', iteration_diag
  write(105, '(A11,I6)') '% diag f  = ', iteration_diag_heavy
  write(105, '(A11,E11.4)') '% t_laser = ', t_laser
  write(105, '(A11,E11.4)') '% E_laser = ', E_laser
  write(105, '(A11,E11.4)') '% C_abs   = ', C_abs
  write(105, '(A11,E11.4)') '% w_laser = ', w_laser
  write(105, '(A11,E11.4)') '% d_laser = ', d_laser
  write(105, '(A11,E11.4)') '% d_targe = ', d_targe
  write(105, '(A11,E11.4)') '% e_targe = ', e_targe
  write(105, '(A12,L1)')    '% T_isol  = ', target_isolated 
  write(file_name , '(A14,I4.4,A4)') 'output_f_Jre__',g,'.txt'
  open(106, file=file_name, status = 'new', form = 'formatted')
  write(106, '(A11,I4)') '% metal   = ', metal
  write(106, '(A11,I6)') '% N_iks   = ', N_iks 
  write(106, '(A11,E11.4)') '% dt      = ', dt
  write(106, '(A11,I6)') '% Nb Th   = ', Nb_Th
  write(106, '(A11,I6)') '% diag s  = ', iteration_diag
  write(106, '(A11,I6)') '% diag f  = ', iteration_diag_heavy
  write(106, '(A11,E11.4)') '% t_laser = ', t_laser
  write(106, '(A11,E11.4)') '% E_laser = ', E_laser
  write(106, '(A11,E11.4)') '% C_abs   = ', C_abs
  write(106, '(A11,E11.4)') '% w_laser = ', w_laser
  write(106, '(A11,E11.4)') '% d_laser = ', d_laser
  write(106, '(A11,E11.4)') '% d_targe = ', d_targe
  write(106, '(A11,E11.4)') '% e_targe = ', e_targe
  write(106, '(A12,L1)')    '% T_isol  = ', target_isolated 
end if

end subroutine create_output_file
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subroutine read_input_file(g)
use parameters
implicit none
LOGICAL :: file_exists=.true.
integer, intent(in) :: g
character(len=14) :: file_name
character(len=11) :: trash
!read input file and fills data for the run.
write(file_name , '(A6,I4.4,A4)') 'input_',g,'.txt'
open(1, file=file_name, status = 'old', form = 'formatted')

read(1, '(A11,I4)')    trash, metal
read(1, '(A11,I6)')    trash, N_iks 
read(1, '(A11,E11.4)') trash, dt
read(1, '(A11,I6)')    trash, Nb_Th
read(1, '(A11,I6)')    trash, iteration_diag
read(1, '(A11,I6)')    trash, iteration_diag_heavy
read(1, '(A11,E11.4)') trash, t_laser
read(1, '(A11,E11.4)') trash, E_laser
read(1, '(A11,E11.4)') trash, C_abs
read(1, '(A11,E11.4)') trash, w_laser
read(1, '(A11,E11.4)') trash, d_laser
read(1, '(A11,E11.4)') trash, d_targe
read(1, '(A11,E11.4)') trash, e_targe
read(1, '(A12,L1)')    trash, target_isolated 
end subroutine read_input_file
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
end program charge_target

